<?php

declare(strict_types=1);

namespace Drupal\graphql_compose\Wrapper;

use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Config\Entity\ConfigEntityInterface;
use Drupal\Core\Config\Entity\ConfigEntityTypeInterface;
use Drupal\Core\Entity\EntityFieldManagerInterface;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Language\LanguageManagerInterface;
use Drupal\Core\StringTranslation\StringTranslationTrait;
use Drupal\graphql_compose\LanguageInflector;
use Drupal\graphql_compose\Plugin\GraphQLCompose\GraphQLComposeEntityTypeInterface;
use Drupal\graphql_compose\Plugin\GraphQLComposeFieldTypeManager as PluginGraphQLComposeFieldTypeManager;
use Drupal\language\Entity\ContentLanguageSettings;
use function Symfony\Component\String\u;

/**
 * Wrapper for an entity.
 */
class EntityTypeWrapper {

  use StringTranslationTrait;

  /**
   * The wrapped entity type plugin.
   *
   * @var mixed
   */
  protected mixed $entity;

  /**
   * The wrapped entity type plugin.
   *
   * @var \Drupal\graphql_compose\Plugin\GraphQLCompose\GraphQLComposeEntityTypeInterface
   */
  protected GraphQLComposeEntityTypeInterface $entityTypePlugin;

  /**
   * Constructs a EntityTypeWrapper object.
   *
   * @param \Drupal\Core\Config\ConfigFactoryInterface $configFactory
   *   The config factory service.
   * @param \Drupal\Core\Entity\EntityFieldManagerInterface $entityFieldManager
   *   The entity field manager service.
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entityTypeManager
   *   The entity type manager service.
   * @param \Drupal\graphql_compose\Plugin\GraphQLComposeFieldTypeManager $gqlFieldTypeManager
   *   The GraphQL Compose field type manager service.
   * @param \Drupal\graphql_compose\LanguageInflector $inflector
   *   The language inflector service.
   * @param \Drupal\Core\Language\LanguageManagerInterface $languageManager
   *   The language manager service.
   */
  public function __construct(
    protected ConfigFactoryInterface $configFactory,
    protected EntityFieldManagerInterface $entityFieldManager,
    protected EntityTypeManagerInterface $entityTypeManager,
    protected PluginGraphQLComposeFieldTypeManager $gqlFieldTypeManager,
    protected LanguageInflector $inflector,
    protected LanguageManagerInterface $languageManager,
  ) {}

  /**
   * Set the wrapped entity.
   *
   * @param mixed $entity
   *   The entity to wrap.
   *
   * @return self
   *   The current instance.
   */
  public function setEntity(mixed $entity): self {
    $this->entity = $entity;
    return $this;
  }

  /**
   * Get the wrapped entity.
   *
   * @return mixed
   *   The wrapped entity.
   */
  public function getEntity(): mixed {
    return $this->entity;
  }

  /**
   * Set the wrapped entity type plugin.
   *
   * @param \Drupal\graphql_compose\Plugin\GraphQLCompose\GraphQLComposeEntityTypeInterface $entityTypePlugin
   *   The wrapped entity type plugin.
   *
   * @return self
   *   The current instance.
   */
  public function setEntityTypePlugin(GraphQLComposeEntityTypeInterface $entityTypePlugin): self {
    $this->entityTypePlugin = $entityTypePlugin;
    return $this;
  }

  /**
   * Get the wrapped entity type plugin.
   *
   * @return \Drupal\graphql_compose\Plugin\GraphQLCompose\GraphQLComposeEntityTypeInterface
   *   The wrapped entity type plugin.
   */
  public function getEntityTypePlugin(): GraphQLComposeEntityTypeInterface {
    return $this->entityTypePlugin;
  }

  /**
   * Type of this entity and bundle.
   *
   * @return string
   *   The GraphQL type name of the entity type. Eg paragraphText.
   *
   * @see hook_graphql_compose_singularize_alter()
   */
  public function getNameSdl(): string {
    $singular = $this->inflector->singularize($this->entity->id());

    return u($singular)
      ->title()
      ->prepend($this->entityTypePlugin->getPrefix())
      ->camel()
      ->toString();
  }

  /**
   * Type of this entity and bundle, plural. Eg paragraphTexts.
   *
   * Some words are the same in singular and plural.
   * Eg News, Sheep. Add ...Items and hope for the best.
   *
   * @return string
   *   The GraphQL type name of the entity type, plural.
   *
   * @see hook_graphql_compose_pluralize_alter()
   */
  public function getNamePluralSdl(): string {
    $singular = $this->inflector->singularize($this->entity->id());
    $plural = $this->inflector->pluralize($singular);

    return u($plural)
      ->title()
      ->prepend($this->entityTypePlugin->getPrefix())
      ->camel()
      ->toString();
  }

  /**
   * Type for the Schema. Title cased singular.
   *
   * @return string
   *   The GraphQL type of the entity type. Eg ParagraphText.
   */
  public function getTypeSdl(): string {
    return u($this->getNameSdl())
      ->camel()
      ->title()
      ->toString();
  }

  /**
   * Return the bundle description or the defined description on the plugin.
   *
   * @return string|null
   *   The description of the wrapped entity.
   */
  public function getDescription(): ?string {
    return method_exists($this->entity, 'getDescription')
      ? $this->entity->getDescription()
      : NULL;
  }

  /**
   * Check if this entity bundle is enabled.
   *
   * @return bool
   *   True if the bundle is enabled.
   */
  public function isEnabled(): bool {
    if ($this->entity instanceof ConfigEntityTypeInterface) {
      return TRUE;
    }

    return (bool) $this->getSetting('enabled') ?: FALSE;
  }

  /**
   * Enabled single resolution query for type. Eg nodePage()
   *
   * @return bool
   *   True if the query load option is enabled.
   */
  public function isQueryLoadEnabled(): bool {
    return (bool) $this->getSetting('query_load_enabled') ?: FALSE;
  }

  /**
   * Check if the bundle has multiple and enabled translations.
   *
   * @return bool
   *   True if we would want to use this as a translation.
   */
  public function isTranslatableContent(): bool {
    // If the site is not multilingual, we don't need to check further.
    if (!$this->languageManager->isMultilingual()) {
      return FALSE;
    }

    // If the entity type does not have a path field, it's no concern.
    if (!array_key_exists('path', $this->entityTypePlugin->getBaseFields())) {
      return FALSE;
    }

    $entity_type = $base_type = $this->entity;
    if ($entity_type instanceof ConfigEntityInterface) {
      if ($bundle_of = $entity_type->getEntityType()->getBundleOf()) {
        $base_type = $this->entityTypeManager->getDefinition($bundle_of);
      }
    }

    // Check if the entity bundle is translatable.
    $config = ContentLanguageSettings::loadByEntityTypeBundle(
      $base_type->id(),
      $entity_type->id()
    );

    return $config->isLanguageAlterable();
  }

  /**
   * Get a config setting.
   *
   * @param string $setting
   *   The setting to get.
   *
   * @return mixed
   *   The setting value or null.
   */
  public function getSetting(string $setting): mixed {
    $settings = $this->configFactory->get('graphql_compose.settings');

    $entity_type = $base_type = $this->entity;
    if ($entity_type instanceof ConfigEntityInterface) {
      if ($bundle_of = $entity_type->getEntityType()->getBundleOf()) {
        $base_type = $this->entityTypeManager->getDefinition($bundle_of);
      }
    }

    return $settings->get('entity_config.' . $base_type->id() . '.' . $entity_type->id() . '.' . $setting) ?: NULL;
  }

}
