<?php

declare(strict_types=1);

namespace Drupal\Tests\graphql_compose\Functional\Core;

use Drupal\menu_link_content\Entity\MenuLinkContent;
use Drupal\node\NodeInterface;
use Drupal\system\Entity\Menu;
use Drupal\system\MenuInterface;
use Drupal\Tests\graphql_compose\Functional\GraphQLComposeBrowserTestBase;
use Drupal\Tests\language\Traits\LanguageTestTrait;

/**
 * Test the entity languages are loading as expected.
 *
 * @group graphql_compose
 */
class EntityLanguageTest extends GraphQLComposeBrowserTestBase {

  use LanguageTestTrait;

  /**
   * The test node.
   *
   * @var \Drupal\node\NodeInterface
   */
  protected NodeInterface $node;

  /**
   * The test menu.
   *
   * @var \Drupal\system\MenuInterface
   */
  protected MenuInterface $menu;

  /**
   * {@inheritdoc}
   */
  protected static $modules = [
    'graphql_compose_menus',
    'graphql_compose_routes',
    'content_translation',
    'config_translation',
    'menu_link_content',
    'language',
  ];

  /**
   * {@inheritdoc}
   */
  protected function setUp(): void {
    parent::setUp();

    $this->createContentType([
      'type' => 'test',
      'name' => 'Test node type',
    ]);

    $this->createLanguageFromLangcode('ja');
    $this->createLanguageFromLangcode('de');

    $this->enableBundleTranslation('node', 'test');

    $this->menu = Menu::create([
      'id' => 'test',
      'label' => 'Test Menu',
    ]);
    $this->menu->save();

    $this->node = $this->createNode([
      'type' => 'test',
      'title' => 'Test',
      'status' => 1,
      'promote' => 1,
      'sticky' => 0,
      'langcode' => 'en',
      'path' => [
        'alias' => '/test',
      ],
    ]);

    $this->node->addTranslation('ja', [
      'title' => 'Test (JA)',
      'path' => [
        'alias' => '/test',
      ],
    ])->save();

    $this->node->addTranslation('de', [
      'title' => 'Test (DE)',
      'path' => [
        'alias' => '/test',
      ],
    ])->save();

    $this->setEntityConfig('menu', 'test', [
      'enabled' => TRUE,
    ]);

    $this->setEntityConfig('node', 'test', [
      'enabled' => TRUE,
      'query_load_enabled' => TRUE,
      'routes_enabled' => TRUE,
    ]);
  }

  /**
   * Test load entity by id.
   */
  public function testNodeLoadByUuid(): void {
    $query = <<<GQL
      query {
        default: node(id: "{$this->node->uuid()}") {
          ... on NodeInterface {
            title
            langcode {
              id
            }
          }
        }

        en: node(id: "{$this->node->uuid()}", langcode: "en") {
          ... on NodeInterface {
            title
            langcode {
              id
            }
          }
        }

        ja: node(id: "{$this->node->uuid()}", langcode: "ja") {
          ... on NodeInterface {
            title
            langcode {
              id
            }
          }
        }

        de: node(id: "{$this->node->uuid()}", langcode: "de") {
          ... on NodeInterface {
            title
            langcode {
              id
            }
          }
        }
      }
    GQL;

    $content = $this->executeQuery($query);

    $default = $content['data']['default'];
    $this->assertEquals('Test', $default['title']);
    $this->assertEquals('en', $default['langcode']['id']);

    $en = $content['data']['en'];
    $this->assertEquals('Test', $en['title']);
    $this->assertEquals('en', $en['langcode']['id']);

    $ja = $content['data']['ja'];
    $this->assertEquals('Test (JA)', $ja['title']);
    $this->assertEquals('ja', $ja['langcode']['id']);

    $de = $content['data']['de'];
    $this->assertEquals('Test (DE)', $de['title']);
    $this->assertEquals('de', $de['langcode']['id']);
  }

  /**
   * Test load entity by route (language).
   */
  public function testRouteLoadWithLangcode(): void {

    $query = <<<GQL
      query {
        en: route(path: "/test", langcode: "en") {
          ... on RouteInternal {
            entity {
              ... on NodeInterface {
                title
                langcode {
                  id
                }
              }
            }
          }
        }

        ja: route(path: "/test", langcode: "ja") {
          ... on RouteInternal {
            entity {
              ... on NodeInterface {
                title
                langcode {
                  id
                }
              }
            }
          }
        }

        de: route(path: "/test", langcode: "de") {
          ... on RouteInternal {
            entity {
              ... on NodeInterface {
                title
                langcode {
                  id
                }
              }
            }
          }
        }
      }
    GQL;

    $content = $this->executeQuery($query);

    $this->assertEquals('Test', $content['data']['en']['entity']['title']);
    $this->assertEquals('en', $content['data']['en']['entity']['langcode']['id']);

    $this->assertEquals('Test (JA)', $content['data']['ja']['entity']['title']);
    $this->assertEquals('ja', $content['data']['ja']['entity']['langcode']['id']);

    $this->assertEquals('Test (DE)', $content['data']['de']['entity']['title']);
    $this->assertEquals('de', $content['data']['de']['entity']['langcode']['id']);
  }

  /**
   * Test load a menu by name with langcode.
   */
  public function testMenuLoadWithLangcode(): void {

    $link = MenuLinkContent::create([
      'title' => 'Test link',
      'link' => ['uri' => 'entity:node/' . $this->node->id()],
      'menu_name' => 'test',
      'langcode' => 'en',
      'default_langcode' => TRUE,
    ]);

    $link->save();

    $link->addTranslation('ja', [
      'title' => 'Test link (JA)',
    ])->save();

    $link->addTranslation('de', [
      'title' => 'Test link (DE)',
    ])->save();

    // Langcode on menu will change the entire response.
    // Each menu needs to be requested separately.
    $query = <<<GQL
      query {
        menu(name: TEST, langcode: "en") {
          items {
            title
            url
          }
        }
      }
    GQL;

    $content = $this->executeQuery($query);

    $this->assertEquals('Test link', $content['data']['menu']['items'][0]['title']);

    // JP.
    $query = <<<GQL
      query {
        menu(name: TEST, langcode: "ja") {
          items {
            title
            url
          }
        }
      }
    GQL;

    $content = $this->executeQuery($query);

    $this->assertEquals('Test link (JA)', $content['data']['menu']['items'][0]['title']);

    // DE.
    $query = <<<GQL
      query {
        menu(name: TEST, langcode: "de") {
          items {
            title
            url
          }
        }
      }
    GQL;

    $content = $this->executeQuery($query);

    $this->assertEquals('Test link (DE)', $content['data']['menu']['items'][0]['title']);
  }

}
